#include "ell/StateCollector.hh"

#include <cmath>

namespace ell
{
	
	SC_PartitionFunction::SC_PartitionFunction(const double _kT)
	  :	SC_Counting(), kT(_kT), Z(0.0)
	{
	}
	
	SC_PartitionFunction::~SC_PartitionFunction()
	{
	}

	
	void
	SC_PartitionFunction::
	add(const State& s)
	{
		  // increase the partition function
		Z += exp( - s.getEnergy() / kT );
		  // store the state
		SC_Counting::add(s);
	}

	
	double
	SC_PartitionFunction::
	getZ( void ) const
	{
		return Z;
	}
	
	double
	SC_PartitionFunction::
	getEnergy( void ) const
	{
		return - kT * log( Z );
	}
	

}



namespace ell
{
	
	SC_PartitionFunction_Shifted::
	SC_PartitionFunction_Shifted( 	const LandscapeTopology& lt,
									const double _kT)
	  :	SC_Counting()
	  	, kT(_kT)
	  	, Z_shifted(0.0)
	  	, MIN_E( lt.getMFEState()->getEnergy() )
	{
	}
	
	SC_PartitionFunction_Shifted::
	SC_PartitionFunction_Shifted( 	const double MinE,
									const double _kT)
	  :	kT(_kT)
		, Z_shifted(0.0)
	  	, MIN_E( MinE )
	{
	}
	
	SC_PartitionFunction_Shifted::
	~SC_PartitionFunction_Shifted()
	{
	}

	
	void
	SC_PartitionFunction_Shifted::
	add(const State& s)
	{
		assertbiu(s.getEnergy() >= MIN_E
			, "the used MIN_E is HIGHER than the energy of the collected State");
		  // increase the shifted partition function
		Z_shifted += exp( (MIN_E - s.getEnergy()) 
							/ kT );
		  // store the state
		SC_Counting::add(s);
	}

	
	double
	SC_PartitionFunction_Shifted::
	getZ( void ) const
	{
		return Z_shifted / exp( MIN_E / kT );
	}
	
	double
	SC_PartitionFunction_Shifted::
	getEnergy( void ) const
	{
		return MIN_E - kT * log( Z_shifted );
	}
	

}
