
#include "ell/protein/WAC_LP.hh"

#include "ell/protein/S_LP.hh"

#include <biu/assertbiu.hh>

namespace ell {

	WAC_LP_final::
	WAC_LP_final(	const std::string& absMoves
					, const biu::LatticeDescriptor& latDescr)
	  :	absMoveStrings()
	  	, moveAlphabet("")
	{
		  // gather all single move string representations of the lattice
		const biu::Alphabet* const lpa = latDescr.getAlphabet();
		for (size_t i=0; i<lpa->getAlphabetSize(); i++) {
			moveAlphabet += lpa->getString(lpa->getElement(i));
		}
		
		  // get all symmetric structures
		typedef std::set<biu::MoveSequence> MSset;
		MSset symmetric = latDescr.getAllSymmetricSequences(latDescr.getSequence(absMoves));
		  // convert to move strings
		for (MSset::const_iterator it=symmetric.begin();it!=symmetric.end();it++) {
			absMoveStrings.insert(latDescr.getString(*it));
		}
	}
	
	WAC_LP_final::
	~WAC_LP_final()
	{
	}
	
	bool 
	WAC_LP_final::
	abort(const StateCollector* const sc) const
	{
		assertbiu(sc!=NULL, "no state collector given");
		assertbiu(sc->getLastAdded()!=NULL, "no last state in state collector available");
		assertbiu(dynamic_cast<const S_LP*>(sc->getLastAdded())!=NULL,"last state in state collector is no S_LP instance");
		
		  // get string representation
		std::string curMoves = sc->getLastAdded()->toString();
		  // derive absolute move string
		curMoves = curMoves.substr( 0
									, curMoves.find_first_not_of(moveAlphabet));
		
		  // check whether or not this move string is present among the
		  // symmetric ones to check for ...
		return absMoveStrings.find(curMoves) != absMoveStrings.end();
	}



} // namespace ell
