#ifndef DUMMYSTATE_HH_
#define DUMMYSTATE_HH_

#include <biu/Alphabet.hh>
#include <biu/assertbiu.hh>

using namespace ell;

#include <sstream>

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double energy;
		size_t maxNeighors;
		mutable size_t leftNeighbors;
		
	public:
		
		class ItState : public State::NeighborList::ItState {
		public:
			ItState() {}
			
			ItState(const ItState &itstate) {}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _energy, const size_t maxNeighNum_) 
		: 	id(_id)
			, energy(_energy)
			, maxNeighors(maxNeighNum_)
			, leftNeighbors(maxNeighNum_)
		{ } 
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			if (leftNeighbors == 0)
				return NULL;
			--leftNeighbors;
			((DummyState*)elem)->applyMove();
			if ( ((DummyState*)elem)->getEnergy() < 0 ) {
				return NULL;
			}
			return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			if (leftNeighbors == 0)
				return NULL;
			--leftNeighbors;
			if (energy <= 0.0)
				return NULL;
			DummyState* s = new DummyState(id, energy, maxNeighors);
			s->applyMove();
			return s;
		}
	};
	
	static const std::string ID;

	virtual
	const std::string& getID() const {
		return DummyState::ID;
	}

	std::string id;
	double energy;
	size_t maxNeighNum;

	static biu::Alphabet a;

	DummyState(std::string _id, double _energy, const size_t maxNeighNum_) 
	: 	id(_id)
		, energy(_energy)
		, maxNeighNum(maxNeighNum_)
	{ } 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return toString() == state2.toString(); }
	bool operator!= (const State& state2) const { return toString() != state2.toString(); }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	DummyState* clone(State* toFill = NULL) const { 
		if (toFill == NULL) return new DummyState(id, energy, maxNeighNum);
		DummyState* s = static_cast<DummyState*>(toFill);
		s->id = this->id;
		s->energy = this->energy;
		return s;
	}
	State* fromString(const std::string& stringRep) const {
		 // get energy
		size_t delim = stringRep.rfind("#");
		if (delim == stringRep.size() || delim == stringRep.size()-1)
			return NULL;
		std::stringstream ss(stringRep.substr(delim+1));
		int e = INT_MAX;
		ss >>e;
		  // create new object
		State* tmp =  new DummyState(stringRep.substr(0,delim), e, maxNeighNum);
		return tmp;
	}
	std::string	toString() const {
		std::stringstream ss;
		ss <<id <<"#"<<energy;
		return ss.str(); 
	}
	std::string&	toString( std::string& toFill) const {
		std::stringstream ss;
		ss <<id <<"#"<<energy;
		toFill = ss.str();
		return toFill;
	}
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy,maxNeighNum));
	}
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy,maxNeighNum));
	}
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	
	DummyState& applyMove() {
		energy -= 1;
		return *this;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		assertbiu(DummyState::a.isAlphabetString(id), "id is no alphabet string");
		return a.compress(a.getSequence(id));
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		toFill = compress();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		assertbiu(false, "not implemented");
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		assertbiu(false, "not implemented");
		return NULL;
	}


};

biu::Alphabet DummyState::a = biu::Alphabet("State 1234567890",1);
const std::string DummyState::ID = std::string("ell::test::DummyState");

#endif /*DUMMYSTATE_HH_*/
